# -*- coding: utf-8 -*-

from zope.security.management import checkPermission


class PermissionAwareResultSet:
    """Lazily accessed set of objects.
    """
    def __init__(self, uids, uidutil, permission):
        self.uids = uids
        self.uidutil = uidutil
        self.perm = permission

    def __len__(self):
        return len(self.uids)

    def __iter__(self):
        for uid in self.uids:
            obj = self.uidutil.getObject(uid)
            if self.perm is None or checkPermission(self.perm, obj):
                yield obj



# -*- coding: utf-8 -*-

from dolmen.app.search.sets import PermissionAwareResultSet

from dolmen.app.search.interfaces import *
from dolmen.app.search.searchers import *

from zope.interface import moduleProvides
from dolmen.app.search.searchers import ISearchers
from dolmen.app.search.interfaces import ISearchAPI

moduleProvides(ISearchers, ISearchAPI)
__all__ = list(ISearchers) + list(ISearchAPI)



# -*- coding: utf-8 -*-

import os.path
import unittest

from zope.testing import doctest, module
from zope.app.testing import functional
from zope.security.testing import Principal, Participation
from zope.security.management import newInteraction, endInteraction

ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
FunctionalLayer = functional.ZCMLLayer(
    ftesting_zcml, __name__, 'FunctionalLayer', allow_teardown=True
    )

def setUp(test):
    module.setUp(test, 'dolmen.app.search.ftests')
    participation = Participation(Principal('zope.mgr'))
    newInteraction(participation)

def tearDown(test):
    module.tearDown(test)
    endInteraction()

def interfaceDescription(interface):
    for name, attr in interface.namesAndDescriptions():
        print "%s: %s" % (name, attr.getDoc())

def test_suite():
    suite = unittest.TestSuite()
    readme = functional.FunctionalDocFileSuite(
        'README.txt', setUp=setUp, tearDown=tearDown,
        globs={'interfaceDescription': interfaceDescription}
        )
    readme.layer = FunctionalLayer
    suite.addTest(readme)
    return suite



# -*- coding: utf-8 -*-

import grok
from zope.interface import Interface
from zope.component import getUtility
from dolmen.app.layout import Page, master
from dolmen.app.search import ICatalogSearcher


class Search(grok.Viewlet):
    grok.context(Interface)
    grok.name('search.viewlet')
    grok.viewletmanager(master.Top)
    grok.order(10)

    def update(self):
        self.term = self.request.form.get('search_term', u'')


class Results(Page):
    grok.context(Interface)
    grok.name('search.result')
    grok.require("dolmen.content.View")

    def update(self):
        searcher = getUtility(ICatalogSearcher, 'searcher.sitecatalog')
        term = self.request.form.get('search_term', u'')
        if term:
            self.results = searcher.search(term + '*')
        else:
            self.results = []
        self.label = u"Found %i results for %s" % (len(self.results), term)



# -*- coding: utf-8 -*-

import grok
from martian import util

from dolmen.app import security
from dolmen.app.search import ICatalogSearcher
from dolmen.app.search import PermissionAwareResultSet

from zope.component import getUtility
from zope.interface import Interface, Attribute, moduleProvides
from zope.app.intid.interfaces import IIntIds
from zope.app.catalog.interfaces import ICatalog


class SiteCatalogSearcher(grok.GlobalUtility):
    grok.name("searcher.sitecatalog")
    grok.implements(ICatalogSearcher)

    @property
    def catalog(self):
        return getUtility(ICatalog)
    
    def search(self, term, index="searchabletext", permission="zope.View"):
        if not len(term):
            return []
        
        results = []
        catalog = self.catalog

        if not index in catalog:
            raise ValueError("Index %r does not exist" % index)

        if util.check_subclass(permission, grok.Permission):
            permission = grok.name.bind().get(permission)

        uidutil = getUtility(IIntIds)
        results = catalog[index].apply(term)
        return PermissionAwareResultSet(results, uidutil, permission)


class ISearchers(Interface):
    """Provided searchers.
    """
    SiteCatalogSearcher = Attribute(
        "ICatalogSearcher querying the application catalog. It verifies "
        "that the current user has the view right (`dolmen.content.View`).")


__all__ = list(ISearchers)



# -*- coding: utf-8 -*-

from zope.schema import Object
from zope.interface import Interface, Attribute, moduleProvides
from zope.catalog.interfaces import ICatalog


class ISearcher(Interface):
    """Defines a component designed to seek and retrieve an element.
    """
    def search(term, *args, **kwargs):
        """Searches elements matching the given term.
        """


class ICatalogSearcher(ISearcher):
    """Defines a specialized searcher using a catalog.
    """
    catalog = Object(
        title = u"Catalog in which the search has to occur.",
        schema = ICatalog,
        required = True
        )

    def search(term, index, permission="zope.View"):
        """Searches a catalog for elements matching the term.
        The searcher can check a given permission on the objects
        of the result set.
        """


class ISearchAPI(Interface):
    """The public Search API.
    """
    ISearcher = Attribute("A component dedicated to search")
    ICatalogSearcher = Attribute("A specialized ISearcher querying a catalog")


moduleProvides(ISearchAPI)
__all__ = list(ISearchAPI)
